import { InjectionToken, Injectable, Inject, ɵɵdefineInjectable, ɵɵinject, PLATFORM_ID, Optional, NgModule, SkipSelf } from '@angular/core';
import { EMPTY, Subject, merge, combineLatest, isObservable, of } from 'rxjs';
import { switchMapTo, switchAll, catchError, map, scan, startWith, filter, shareReplay, switchMap, takeUntil } from 'rxjs/operators';
import { __rest } from 'tslib';
import { isPlatformServer } from '@angular/common';
import { makeStateKey, Title, Meta, TransferState } from '@angular/platform-browser';
import { NavigationEnd, Router } from '@angular/router';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@internal
 * @type {?}
 */
const ROUTE_META_CONFIG = new InjectionToken('ROUTE_META_CONFIG');
/**
 * Check if {\@link Route#data} contains {\@link RouteMeta} object
 * @param {?} data
 * @return {?}
 */
function isDataWithMeta(data) {
    return !!data.meta;
}
/**
 * Allows to accumulate many context objects streams into one stream
 * while merging all context objects together
 * and running any transformation function
 * @template R
 * @param {?} ctx$$
 * @param {?} mapFn
 * @return {?}
 */
function unfoldContext(ctx$$, mapFn) {
    return (/**
     * @param {?} o$
     * @return {?}
     */
    o$ => o$.pipe(switchMapTo(ctx$$.pipe(switchAll(), catchError((/**
     * @return {?}
     */
    () => EMPTY)), map((/**
     * @param {?} ctx
     * @return {?}
     */
    ctx => mapFn(ctx))), scan((/**
     * @param {?} acc
     * @param {?} ctx
     * @return {?}
     */
    (acc, ctx) => (Object.assign({}, acc, ctx)))), // Merge contexts
    startWith((/** @type {?} */ ({})))))));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} event
 * @return {?}
 */
function isNavigationEndEvent(event) {
    return event instanceof NavigationEnd;
}
/**
 * @param {?} str
 * @return {?}
 */
function escapeRegExp(str) {
    return (/** @type {?} */ (str.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'))); // $& means the whole matched string
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Service to provide context for meta tags
 */
class RouterMetaContextService {
    /**
     * @param {?} config
     * @param {?} router
     */
    constructor(config, router) {
        this.config = config;
        this.router = router;
        this.ctxNameCache = {};
        this.interpolation = this.config.interpolation || RouterMetaContextService.interpolation;
        this.interpolationStart = escapeRegExp(this.interpolation.start);
        this.interpolationEnd = escapeRegExp(this.interpolation.end);
        this.navigationEnd$ = this.router.events.pipe(filter(isNavigationEndEvent));
        this.metaDefaultContext$$ = new Subject();
        this.metaContext$$ = new Subject();
        this.clearDefaultContext$ = new Subject();
        this.clearContext$ = new Subject();
        this.metaDefaultContext$ = this.clearDefaultContext$.pipe(startWith(null), unfoldContext(this.metaDefaultContext$$, (/**
         * @param {?} ctx
         * @return {?}
         */
        ctx => this._processContext(ctx))));
        this.metaContext$ = merge(this.clearContext$, this.navigationEnd$).pipe(startWith(null), unfoldContext(this.metaContext$$, (/**
         * @param {?} ctx
         * @return {?}
         */
        ctx => this._processContext(ctx))));
        this.context$ = combineLatest(this.metaDefaultContext$, this.metaContext$).pipe(map((/**
         * @param {?} __0
         * @return {?}
         */
        ([defaultCtx, ctx]) => (Object.assign({}, defaultCtx, ctx)))), shareReplay());
    }
    /**
     * Provided context will be available between router navigations
     *
     * Multiple contexts will be merged together
     * @param {?} ctx
     * @return {?}
     */
    provideDefaultContext(ctx) {
        this.metaDefaultContext$$.next(isObservable(ctx) ? ctx : of(ctx));
    }
    /**
     * Provided context will be available ONLY until next router navigation
     *
     * Multiple contexts will be merged together
     *
     * If you want to persist context between navigations -
     * use {\@link RouterMetaContextService#provideDefaultContext()}
     * @param {?} ctx
     * @return {?}
     */
    provideContext(ctx) {
        this.metaContext$$.next(isObservable(ctx) ? ctx : of(ctx));
    }
    /**
     * Clear default context provided by
     * {\@link RouterMetaContextService#provideDefaultContext()}
     * @return {?}
     */
    clearDefaultContext() {
        this.clearDefaultContext$.next();
    }
    /**
     * Clear context provided by
     * {\@link RouterMetaContextService#provideContext()}
     * @return {?}
     */
    clearContext() {
        this.clearContext$.next();
    }
    /**
     * Observable of computed context for meta tags
     * @return {?}
     */
    getContext() {
        return this.context$;
    }
    /**
     * \@internal
     * @param {?=} str
     * @param {?=} data
     * @param {?=} extras
     * @return {?}
     */
    _templateStr(str, data, extras) {
        if (!str || !data) {
            return str || '';
        }
        /** @type {?} */
        let template = str;
        /** @type {?} */
        let ctx = data;
        if (extras) {
            template =
                extras.templates && extras.name
                    ? extras.templates[extras.name] || template
                    : template;
            ctx = Object.assign({}, this.prepareContext(extras.meta, ctx), ctx);
            // Recover lost `str` value under provided `extras.name` key in context
            if (extras.name && extras.name in ctx === false) {
                /** @type {?} */
                const nameCtx = this._processContext({ [extras.name]: str });
                ctx = Object.assign({}, ctx, this.prepareContext(nameCtx, ctx));
            }
        }
        return this.templateReplace(template, ctx);
    }
    /**
     * \@internal
     * @param {?} ctx
     * @return {?}
     */
    _processContext(ctx) {
        return Object.keys(ctx).reduce((/**
         * @param {?} acc
         * @param {?} key
         * @return {?}
         */
        (acc, key) => (Object.assign({}, acc, { [key]: {
                replace: this.getContextReplace(key),
                value: ctx[key],
            } }))), (/** @type {?} */ ({})));
    }
    /**
     * @private
     * @param {?=} ctx
     * @param {?=} data
     * @return {?}
     */
    prepareContext(ctx, data) {
        if (!ctx || !data) {
            return ctx;
        }
        return Object.keys(ctx).reduce((/**
         * @param {?} c
         * @param {?} key
         * @return {?}
         */
        (c, key) => {
            c[key] = Object.assign({}, ctx[key], { value: this.templateReplace(ctx[key].value, data) });
            return c;
        }), (/** @type {?} */ ({})));
    }
    /**
     * @private
     * @param {?} tpl
     * @param {?} ctx
     * @return {?}
     */
    templateReplace(tpl, ctx) {
        return Object.keys(ctx).reduce((/**
         * @param {?} s
         * @param {?} key
         * @return {?}
         */
        (s, key) => s.replace(ctx[key].replace, ctx[key].value)), tpl);
    }
    /**
     * @private
     * @param {?} name
     * @return {?}
     */
    getContextReplace(name) {
        return name in this.ctxNameCache
            ? this.ctxNameCache[name]
            : (this.ctxNameCache[name] = new RegExp(`${this.interpolationStart}${escapeRegExp(name)}${this.interpolationEnd}`, 'g'));
    }
}
RouterMetaContextService.interpolation = {
    start: '{',
    end: '}',
};
RouterMetaContextService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];
/** @nocollapse */
RouterMetaContextService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [ROUTE_META_CONFIG,] }] },
    { type: Router }
];
/** @nocollapse */ RouterMetaContextService.ngInjectableDef = ɵɵdefineInjectable({ factory: function RouterMetaContextService_Factory() { return new RouterMetaContextService(ɵɵinject(ROUTE_META_CONFIG), ɵɵinject(Router)); }, token: RouterMetaContextService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const TITLE_STATE = makeStateKey('RouterMetaTitle');
/**
 * Service responsible for updating meta tags based on router configuration
 *
 * To provide context - use {\@link RouterMetaContextService}
 */
class RouterMetaService {
    /**
     * @param {?} platformId
     * @param {?} config
     * @param {?} router
     * @param {?} title
     * @param {?} meta
     * @param {?} contextService
     * @param {?=} transferState
     */
    constructor(platformId, config, router, title, meta, contextService, transferState) {
        this.platformId = platformId;
        this.config = config;
        this.router = router;
        this.title = title;
        this.meta = meta;
        this.contextService = contextService;
        this.transferState = transferState;
        this.initialized = false;
        this.metaTags = {};
        this.defaultMeta = this.config.defaultMeta || {};
        this.originalTitle = this.transferState
            ? this.transferState.get(TITLE_STATE, this.title.getTitle())
            : this.title.getTitle();
        this.destroyed$ = new Subject();
        this.navigationEnd$ = this.router.events.pipe(filter(isNavigationEndEvent));
        this.routeData$ = this.navigationEnd$.pipe(map((/**
         * @return {?}
         */
        () => this.router.routerState.root)), map((/**
         * @param {?} route
         * @return {?}
         */
        route => this.getLastRoute(route))), filter((/**
         * @param {?} route
         * @return {?}
         */
        route => route.outlet === 'primary')), switchMap((/**
         * @param {?} route
         * @return {?}
         */
        route => route.data)));
    }
    /**
     * Returns original title of index page.
     *
     * If SSR was used with {\@link StateTransfer} -
     * then original title will be preserved between server and client states
     * @return {?}
     */
    getOriginalTitle() {
        return this.originalTitle;
    }
    /**
     * \@internal
     * @return {?}
     */
    ngOnDestroy() {
        this.destroyed$.next();
    }
    /**
     * \@internal
     * @return {?}
     */
    _setup() {
        if (this.initialized) {
            return;
        }
        this.initialized = true;
        if (this.transferState && isPlatformServer(this.platformId)) {
            this.transferState.set(TITLE_STATE, this.originalTitle);
        }
        combineLatest(this.routeData$, this.contextService.getContext())
            .pipe(takeUntil(this.destroyed$))
            .subscribe((/**
         * @param {?} __0
         * @return {?}
         */
        ([data, ctx]) => this.updateAllMeta(data, ctx)));
    }
    /**
     * @private
     * @param {?} data
     * @param {?=} ctx
     * @return {?}
     */
    updateAllMeta(data, ctx = {}) {
        const _a = this.defaultMeta, { _templates_: defaultTemplates } = _a, defaultAllMeta = __rest(_a, ["_templates_"]);
        if (!isDataWithMeta(data)) {
            this.resetAllMeta(ctx, defaultAllMeta);
            return;
        }
        const { meta } = data;
        const { _templates_ } = meta, allMeta = __rest(meta, ["_templates_"]);
        const { title } = allMeta, otherMeta = __rest(allMeta, ["title"]);
        const defaultOtherMeta = __rest(defaultAllMeta, ["title"]);
        /** @type {?} */
        const allMetaDefaulted = Object.assign({}, defaultAllMeta, allMeta);
        /** @type {?} */
        const otherMetaDefaulted = Object.assign({}, defaultOtherMeta, otherMeta);
        /** @type {?} */
        const templates = (/** @type {?} */ (Object.assign({}, defaultTemplates, _templates_)));
        /** @type {?} */
        const processedMeta = this.contextService._processContext(allMetaDefaulted);
        this.updateTitle(title, ctx, templates, processedMeta);
        Object.keys(otherMetaDefaulted).forEach((/**
         * @param {?} name
         * @return {?}
         */
        name => this.updateMeta(name, otherMetaDefaulted[name], ctx, templates, processedMeta)));
        // Cleanup leftover meta tags
        Object.keys(this.metaTags)
            .filter((/**
         * @param {?} name
         * @return {?}
         */
        name => name in otherMetaDefaulted === false))
            .forEach((/**
         * @param {?} name
         * @return {?}
         */
        name => this.resetMeta(name, ctx, templates, processedMeta)));
    }
    /**
     * @private
     * @param {?} ctx
     * @param {?} defaultMeta
     * @return {?}
     */
    resetAllMeta(ctx, defaultMeta) {
        const defaultOtherMeta = __rest(defaultMeta, ["title"]);
        /** @type {?} */
        const metaDefaulted = Object.assign({}, defaultOtherMeta, this.metaTags);
        /** @type {?} */
        const meta = this.contextService._processContext(defaultMeta);
        this.resetTitle(ctx, this.defaultMeta._templates_, meta);
        Object.keys(metaDefaulted).forEach((/**
         * @param {?} name
         * @return {?}
         */
        name => this.resetMeta(name, ctx, this.defaultMeta._templates_, meta)));
    }
    /**
     * @private
     * @param {?=} title
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    updateTitle(title, ctx, templates, meta) {
        if (title) {
            this.title.setTitle(this.contextService._templateStr(title, ctx, {
                name: 'title',
                templates,
                meta,
            }));
        }
        else {
            this.resetTitle(ctx, templates, meta);
        }
    }
    /**
     * @private
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    resetTitle(ctx, templates, meta) {
        this.title.setTitle(this.contextService._templateStr(this.defaultMeta.title || this.originalTitle, ctx, {
            name: 'title',
            meta,
            templates,
        }));
    }
    /**
     * @private
     * @param {?} name
     * @param {?=} content
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    updateMeta(name, content, ctx, templates, meta) {
        if (content) {
            /** @type {?} */
            const metaDef = typeof content === 'string' ? { name, content } : content;
            metaDef.content = this.contextService._templateStr(metaDef.content, ctx, {
                name,
                meta,
                templates,
            });
            this.meta.updateTag(metaDef);
            this.metaTags[name] = true;
        }
        else {
            this.resetMeta(name, ctx, templates, meta);
        }
    }
    /**
     * @private
     * @param {?} name
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    resetMeta(name, ctx, templates, meta) {
        /** @type {?} */
        const defaultMeta = this.defaultMeta[name];
        if (defaultMeta) {
            this.updateMeta(name, defaultMeta, ctx, templates, meta);
        }
        else {
            this.meta.removeTag(`name="${name}"`);
            delete this.metaTags[name];
        }
    }
    /**
     * @private
     * @param {?} route
     * @return {?}
     */
    getLastRoute(route) {
        while (route.firstChild) {
            route = route.firstChild;
        }
        return route;
    }
}
RouterMetaService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
RouterMetaService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [ROUTE_META_CONFIG,] }] },
    { type: Router },
    { type: Title },
    { type: Meta },
    { type: RouterMetaContextService },
    { type: TransferState, decorators: [{ type: Optional }] }
];
/** @nocollapse */ RouterMetaService.ngInjectableDef = ɵɵdefineInjectable({ factory: function RouterMetaService_Factory() { return new RouterMetaService(ɵɵinject(PLATFORM_ID), ɵɵinject(ROUTE_META_CONFIG), ɵɵinject(Router), ɵɵinject(Title), ɵɵinject(Meta), ɵɵinject(RouterMetaContextService), ɵɵinject(TransferState, 8)); }, token: RouterMetaService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RouterMetaModule {
    /**
     * @param {?} parentModule
     * @param {?} routerMetaService
     */
    constructor(parentModule, routerMetaService) {
        if (!parentModule) {
            routerMetaService._setup();
        }
    }
    /**
     * @template C
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config = (/** @type {?} */ ({}))) {
        return {
            ngModule: RouterMetaModule,
            providers: [
                config.configProvider || {
                    provide: ROUTE_META_CONFIG,
                    useValue: config.config,
                },
            ],
        };
    }
}
RouterMetaModule.decorators = [
    { type: NgModule, args: [{},] }
];
/** @nocollapse */
RouterMetaModule.ctorParameters = () => [
    { type: RouterMetaModule, decorators: [{ type: Optional }, { type: SkipSelf }] },
    { type: RouterMetaService }
];

export { ROUTE_META_CONFIG, RouterMetaContextService, RouterMetaModule, RouterMetaService, isDataWithMeta, unfoldContext };
//# sourceMappingURL=ngx-router-meta.js.map
