import { __assign, __read, __rest } from 'tslib';
import { InjectionToken, Injectable, Inject, ɵɵdefineInjectable, ɵɵinject, PLATFORM_ID, Optional, NgModule, SkipSelf } from '@angular/core';
import { EMPTY, isObservable, of, Subject, merge, combineLatest } from 'rxjs';
import { switchMapTo, switchAll, catchError, map, scan, startWith, filter, shareReplay, takeUntil, switchMap } from 'rxjs/operators';
import { isPlatformServer } from '@angular/common';
import { makeStateKey, Title, Meta, TransferState } from '@angular/platform-browser';
import { NavigationEnd, Router } from '@angular/router';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@internal
 * @type {?}
 */
var ROUTE_META_CONFIG = new InjectionToken('ROUTE_META_CONFIG');
/**
 * Check if {\@link Route#data} contains {\@link RouteMeta} object
 * @param {?} data
 * @return {?}
 */
function isDataWithMeta(data) {
    return !!data.meta;
}
/**
 * Allows to accumulate many context objects streams into one stream
 * while merging all context objects together
 * and running any transformation function
 * @template R
 * @param {?} ctx$$
 * @param {?} mapFn
 * @return {?}
 */
function unfoldContext(ctx$$, mapFn) {
    return (/**
     * @param {?} o$
     * @return {?}
     */
    function (o$) {
        return o$.pipe(switchMapTo(ctx$$.pipe(switchAll(), catchError((/**
         * @return {?}
         */
        function () { return EMPTY; })), map((/**
         * @param {?} ctx
         * @return {?}
         */
        function (ctx) { return mapFn(ctx); })), scan((/**
         * @param {?} acc
         * @param {?} ctx
         * @return {?}
         */
        function (acc, ctx) { return (__assign({}, acc, ctx)); })), // Merge contexts
        startWith((/** @type {?} */ ({}))))));
    });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} event
 * @return {?}
 */
function isNavigationEndEvent(event) {
    return event instanceof NavigationEnd;
}
/**
 * @param {?} str
 * @return {?}
 */
function escapeRegExp(str) {
    return (/** @type {?} */ (str.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'))); // $& means the whole matched string
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Service to provide context for meta tags
 */
var RouterMetaContextService = /** @class */ (function () {
    function RouterMetaContextService(config, router) {
        var _this = this;
        this.config = config;
        this.router = router;
        this.ctxNameCache = {};
        this.interpolation = this.config.interpolation || RouterMetaContextService.interpolation;
        this.interpolationStart = escapeRegExp(this.interpolation.start);
        this.interpolationEnd = escapeRegExp(this.interpolation.end);
        this.navigationEnd$ = this.router.events.pipe(filter(isNavigationEndEvent));
        this.metaDefaultContext$$ = new Subject();
        this.metaContext$$ = new Subject();
        this.clearDefaultContext$ = new Subject();
        this.clearContext$ = new Subject();
        this.metaDefaultContext$ = this.clearDefaultContext$.pipe(startWith(null), unfoldContext(this.metaDefaultContext$$, (/**
         * @param {?} ctx
         * @return {?}
         */
        function (ctx) { return _this._processContext(ctx); })));
        this.metaContext$ = merge(this.clearContext$, this.navigationEnd$).pipe(startWith(null), unfoldContext(this.metaContext$$, (/**
         * @param {?} ctx
         * @return {?}
         */
        function (ctx) { return _this._processContext(ctx); })));
        this.context$ = combineLatest(this.metaDefaultContext$, this.metaContext$).pipe(map((/**
         * @param {?} __0
         * @return {?}
         */
        function (_a) {
            var _b = __read(_a, 2), defaultCtx = _b[0], ctx = _b[1];
            return (__assign({}, defaultCtx, ctx));
        })), shareReplay());
    }
    /**
     * Provided context will be available between router navigations
     *
     * Multiple contexts will be merged together
     */
    /**
     * Provided context will be available between router navigations
     *
     * Multiple contexts will be merged together
     * @param {?} ctx
     * @return {?}
     */
    RouterMetaContextService.prototype.provideDefaultContext = /**
     * Provided context will be available between router navigations
     *
     * Multiple contexts will be merged together
     * @param {?} ctx
     * @return {?}
     */
    function (ctx) {
        this.metaDefaultContext$$.next(isObservable(ctx) ? ctx : of(ctx));
    };
    /**
     * Provided context will be available ONLY until next router navigation
     *
     * Multiple contexts will be merged together
     *
     * If you want to persist context between navigations -
     * use {@link RouterMetaContextService#provideDefaultContext()}
     */
    /**
     * Provided context will be available ONLY until next router navigation
     *
     * Multiple contexts will be merged together
     *
     * If you want to persist context between navigations -
     * use {\@link RouterMetaContextService#provideDefaultContext()}
     * @param {?} ctx
     * @return {?}
     */
    RouterMetaContextService.prototype.provideContext = /**
     * Provided context will be available ONLY until next router navigation
     *
     * Multiple contexts will be merged together
     *
     * If you want to persist context between navigations -
     * use {\@link RouterMetaContextService#provideDefaultContext()}
     * @param {?} ctx
     * @return {?}
     */
    function (ctx) {
        this.metaContext$$.next(isObservable(ctx) ? ctx : of(ctx));
    };
    /**
     * Clear default context provided by
     * {@link RouterMetaContextService#provideDefaultContext()}
     */
    /**
     * Clear default context provided by
     * {\@link RouterMetaContextService#provideDefaultContext()}
     * @return {?}
     */
    RouterMetaContextService.prototype.clearDefaultContext = /**
     * Clear default context provided by
     * {\@link RouterMetaContextService#provideDefaultContext()}
     * @return {?}
     */
    function () {
        this.clearDefaultContext$.next();
    };
    /**
     * Clear context provided by
     * {@link RouterMetaContextService#provideContext()}
     */
    /**
     * Clear context provided by
     * {\@link RouterMetaContextService#provideContext()}
     * @return {?}
     */
    RouterMetaContextService.prototype.clearContext = /**
     * Clear context provided by
     * {\@link RouterMetaContextService#provideContext()}
     * @return {?}
     */
    function () {
        this.clearContext$.next();
    };
    /**
     * Observable of computed context for meta tags
     */
    /**
     * Observable of computed context for meta tags
     * @return {?}
     */
    RouterMetaContextService.prototype.getContext = /**
     * Observable of computed context for meta tags
     * @return {?}
     */
    function () {
        return this.context$;
    };
    /** @internal */
    /**
     * \@internal
     * @param {?=} str
     * @param {?=} data
     * @param {?=} extras
     * @return {?}
     */
    RouterMetaContextService.prototype._templateStr = /**
     * \@internal
     * @param {?=} str
     * @param {?=} data
     * @param {?=} extras
     * @return {?}
     */
    function (str, data, extras) {
        var _a;
        if (!str || !data) {
            return str || '';
        }
        /** @type {?} */
        var template = str;
        /** @type {?} */
        var ctx = data;
        if (extras) {
            template =
                extras.templates && extras.name
                    ? extras.templates[extras.name] || template
                    : template;
            ctx = __assign({}, this.prepareContext(extras.meta, ctx), ctx);
            // Recover lost `str` value under provided `extras.name` key in context
            if (extras.name && extras.name in ctx === false) {
                /** @type {?} */
                var nameCtx = this._processContext((_a = {}, _a[extras.name] = str, _a));
                ctx = __assign({}, ctx, this.prepareContext(nameCtx, ctx));
            }
        }
        return this.templateReplace(template, ctx);
    };
    /** @internal */
    /**
     * \@internal
     * @param {?} ctx
     * @return {?}
     */
    RouterMetaContextService.prototype._processContext = /**
     * \@internal
     * @param {?} ctx
     * @return {?}
     */
    function (ctx) {
        var _this = this;
        return Object.keys(ctx).reduce((/**
         * @param {?} acc
         * @param {?} key
         * @return {?}
         */
        function (acc, key) {
            var _a;
            return (__assign({}, acc, (_a = {}, _a[key] = {
                replace: _this.getContextReplace(key),
                value: ctx[key],
            }, _a)));
        }), (/** @type {?} */ ({})));
    };
    /**
     * @private
     * @param {?=} ctx
     * @param {?=} data
     * @return {?}
     */
    RouterMetaContextService.prototype.prepareContext = /**
     * @private
     * @param {?=} ctx
     * @param {?=} data
     * @return {?}
     */
    function (ctx, data) {
        var _this = this;
        if (!ctx || !data) {
            return ctx;
        }
        return Object.keys(ctx).reduce((/**
         * @param {?} c
         * @param {?} key
         * @return {?}
         */
        function (c, key) {
            c[key] = __assign({}, ctx[key], { value: _this.templateReplace(ctx[key].value, data) });
            return c;
        }), (/** @type {?} */ ({})));
    };
    /**
     * @private
     * @param {?} tpl
     * @param {?} ctx
     * @return {?}
     */
    RouterMetaContextService.prototype.templateReplace = /**
     * @private
     * @param {?} tpl
     * @param {?} ctx
     * @return {?}
     */
    function (tpl, ctx) {
        return Object.keys(ctx).reduce((/**
         * @param {?} s
         * @param {?} key
         * @return {?}
         */
        function (s, key) { return s.replace(ctx[key].replace, ctx[key].value); }), tpl);
    };
    /**
     * @private
     * @param {?} name
     * @return {?}
     */
    RouterMetaContextService.prototype.getContextReplace = /**
     * @private
     * @param {?} name
     * @return {?}
     */
    function (name) {
        return name in this.ctxNameCache
            ? this.ctxNameCache[name]
            : (this.ctxNameCache[name] = new RegExp("" + this.interpolationStart + escapeRegExp(name) + this.interpolationEnd, 'g'));
    };
    RouterMetaContextService.interpolation = {
        start: '{',
        end: '}',
    };
    RouterMetaContextService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root',
                },] }
    ];
    /** @nocollapse */
    RouterMetaContextService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [ROUTE_META_CONFIG,] }] },
        { type: Router }
    ]; };
    /** @nocollapse */ RouterMetaContextService.ngInjectableDef = ɵɵdefineInjectable({ factory: function RouterMetaContextService_Factory() { return new RouterMetaContextService(ɵɵinject(ROUTE_META_CONFIG), ɵɵinject(Router)); }, token: RouterMetaContextService, providedIn: "root" });
    return RouterMetaContextService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var TITLE_STATE = makeStateKey('RouterMetaTitle');
/**
 * Service responsible for updating meta tags based on router configuration
 *
 * To provide context - use {\@link RouterMetaContextService}
 */
var RouterMetaService = /** @class */ (function () {
    function RouterMetaService(platformId, config, router, title, meta, contextService, transferState) {
        var _this = this;
        this.platformId = platformId;
        this.config = config;
        this.router = router;
        this.title = title;
        this.meta = meta;
        this.contextService = contextService;
        this.transferState = transferState;
        this.initialized = false;
        this.metaTags = {};
        this.defaultMeta = this.config.defaultMeta || {};
        this.originalTitle = this.transferState
            ? this.transferState.get(TITLE_STATE, this.title.getTitle())
            : this.title.getTitle();
        this.destroyed$ = new Subject();
        this.navigationEnd$ = this.router.events.pipe(filter(isNavigationEndEvent));
        this.routeData$ = this.navigationEnd$.pipe(map((/**
         * @return {?}
         */
        function () { return _this.router.routerState.root; })), map((/**
         * @param {?} route
         * @return {?}
         */
        function (route) { return _this.getLastRoute(route); })), filter((/**
         * @param {?} route
         * @return {?}
         */
        function (route) { return route.outlet === 'primary'; })), switchMap((/**
         * @param {?} route
         * @return {?}
         */
        function (route) { return route.data; })));
    }
    /**
     * Returns original title of index page.
     *
     * If SSR was used with {@link StateTransfer} -
     * then original title will be preserved between server and client states
     */
    /**
     * Returns original title of index page.
     *
     * If SSR was used with {\@link StateTransfer} -
     * then original title will be preserved between server and client states
     * @return {?}
     */
    RouterMetaService.prototype.getOriginalTitle = /**
     * Returns original title of index page.
     *
     * If SSR was used with {\@link StateTransfer} -
     * then original title will be preserved between server and client states
     * @return {?}
     */
    function () {
        return this.originalTitle;
    };
    /** @internal */
    /**
     * \@internal
     * @return {?}
     */
    RouterMetaService.prototype.ngOnDestroy = /**
     * \@internal
     * @return {?}
     */
    function () {
        this.destroyed$.next();
    };
    /** @internal */
    /**
     * \@internal
     * @return {?}
     */
    RouterMetaService.prototype._setup = /**
     * \@internal
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.initialized) {
            return;
        }
        this.initialized = true;
        if (this.transferState && isPlatformServer(this.platformId)) {
            this.transferState.set(TITLE_STATE, this.originalTitle);
        }
        combineLatest(this.routeData$, this.contextService.getContext())
            .pipe(takeUntil(this.destroyed$))
            .subscribe((/**
         * @param {?} __0
         * @return {?}
         */
        function (_a) {
            var _b = __read(_a, 2), data = _b[0], ctx = _b[1];
            return _this.updateAllMeta(data, ctx);
        }));
    };
    /**
     * @private
     * @param {?} data
     * @param {?=} ctx
     * @return {?}
     */
    RouterMetaService.prototype.updateAllMeta = /**
     * @private
     * @param {?} data
     * @param {?=} ctx
     * @return {?}
     */
    function (data, ctx) {
        var _this = this;
        if (ctx === void 0) { ctx = {}; }
        var _a = this.defaultMeta, defaultTemplates = _a._templates_, defaultAllMeta = __rest(_a, ["_templates_"]);
        if (!isDataWithMeta(data)) {
            this.resetAllMeta(ctx, defaultAllMeta);
            return;
        }
        var meta = data.meta;
        var _templates_ = meta._templates_, allMeta = __rest(meta, ["_templates_"]);
        var title = allMeta.title, otherMeta = __rest(allMeta, ["title"]);
        var _ = defaultAllMeta.title, defaultOtherMeta = __rest(defaultAllMeta, ["title"]);
        /** @type {?} */
        var allMetaDefaulted = __assign({}, defaultAllMeta, allMeta);
        /** @type {?} */
        var otherMetaDefaulted = __assign({}, defaultOtherMeta, otherMeta);
        /** @type {?} */
        var templates = (/** @type {?} */ (__assign({}, defaultTemplates, _templates_)));
        /** @type {?} */
        var processedMeta = this.contextService._processContext(allMetaDefaulted);
        this.updateTitle(title, ctx, templates, processedMeta);
        Object.keys(otherMetaDefaulted).forEach((/**
         * @param {?} name
         * @return {?}
         */
        function (name) {
            return _this.updateMeta(name, otherMetaDefaulted[name], ctx, templates, processedMeta);
        }));
        // Cleanup leftover meta tags
        Object.keys(this.metaTags)
            .filter((/**
         * @param {?} name
         * @return {?}
         */
        function (name) { return name in otherMetaDefaulted === false; }))
            .forEach((/**
         * @param {?} name
         * @return {?}
         */
        function (name) { return _this.resetMeta(name, ctx, templates, processedMeta); }));
    };
    /**
     * @private
     * @param {?} ctx
     * @param {?} defaultMeta
     * @return {?}
     */
    RouterMetaService.prototype.resetAllMeta = /**
     * @private
     * @param {?} ctx
     * @param {?} defaultMeta
     * @return {?}
     */
    function (ctx, defaultMeta) {
        var _this = this;
        var title = defaultMeta.title, defaultOtherMeta = __rest(defaultMeta, ["title"]);
        /** @type {?} */
        var metaDefaulted = __assign({}, defaultOtherMeta, this.metaTags);
        /** @type {?} */
        var meta = this.contextService._processContext(defaultMeta);
        this.resetTitle(ctx, this.defaultMeta._templates_, meta);
        Object.keys(metaDefaulted).forEach((/**
         * @param {?} name
         * @return {?}
         */
        function (name) {
            return _this.resetMeta(name, ctx, _this.defaultMeta._templates_, meta);
        }));
    };
    /**
     * @private
     * @param {?=} title
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    RouterMetaService.prototype.updateTitle = /**
     * @private
     * @param {?=} title
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    function (title, ctx, templates, meta) {
        if (title) {
            this.title.setTitle(this.contextService._templateStr(title, ctx, {
                name: 'title',
                templates: templates,
                meta: meta,
            }));
        }
        else {
            this.resetTitle(ctx, templates, meta);
        }
    };
    /**
     * @private
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    RouterMetaService.prototype.resetTitle = /**
     * @private
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    function (ctx, templates, meta) {
        this.title.setTitle(this.contextService._templateStr(this.defaultMeta.title || this.originalTitle, ctx, {
            name: 'title',
            meta: meta,
            templates: templates,
        }));
    };
    /**
     * @private
     * @param {?} name
     * @param {?=} content
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    RouterMetaService.prototype.updateMeta = /**
     * @private
     * @param {?} name
     * @param {?=} content
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    function (name, content, ctx, templates, meta) {
        if (content) {
            /** @type {?} */
            var metaDef = typeof content === 'string' ? { name: name, content: content } : content;
            metaDef.content = this.contextService._templateStr(metaDef.content, ctx, {
                name: name,
                meta: meta,
                templates: templates,
            });
            this.meta.updateTag(metaDef);
            this.metaTags[name] = true;
        }
        else {
            this.resetMeta(name, ctx, templates, meta);
        }
    };
    /**
     * @private
     * @param {?} name
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    RouterMetaService.prototype.resetMeta = /**
     * @private
     * @param {?} name
     * @param {?=} ctx
     * @param {?=} templates
     * @param {?=} meta
     * @return {?}
     */
    function (name, ctx, templates, meta) {
        /** @type {?} */
        var defaultMeta = this.defaultMeta[name];
        if (defaultMeta) {
            this.updateMeta(name, defaultMeta, ctx, templates, meta);
        }
        else {
            this.meta.removeTag("name=\"" + name + "\"");
            delete this.metaTags[name];
        }
    };
    /**
     * @private
     * @param {?} route
     * @return {?}
     */
    RouterMetaService.prototype.getLastRoute = /**
     * @private
     * @param {?} route
     * @return {?}
     */
    function (route) {
        while (route.firstChild) {
            route = route.firstChild;
        }
        return route;
    };
    RouterMetaService.decorators = [
        { type: Injectable, args: [{ providedIn: 'root' },] }
    ];
    /** @nocollapse */
    RouterMetaService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [ROUTE_META_CONFIG,] }] },
        { type: Router },
        { type: Title },
        { type: Meta },
        { type: RouterMetaContextService },
        { type: TransferState, decorators: [{ type: Optional }] }
    ]; };
    /** @nocollapse */ RouterMetaService.ngInjectableDef = ɵɵdefineInjectable({ factory: function RouterMetaService_Factory() { return new RouterMetaService(ɵɵinject(PLATFORM_ID), ɵɵinject(ROUTE_META_CONFIG), ɵɵinject(Router), ɵɵinject(Title), ɵɵinject(Meta), ɵɵinject(RouterMetaContextService), ɵɵinject(TransferState, 8)); }, token: RouterMetaService, providedIn: "root" });
    return RouterMetaService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var RouterMetaModule = /** @class */ (function () {
    function RouterMetaModule(parentModule, routerMetaService) {
        if (!parentModule) {
            routerMetaService._setup();
        }
    }
    /**
     * @template C
     * @param {?=} config
     * @return {?}
     */
    RouterMetaModule.forRoot = /**
     * @template C
     * @param {?=} config
     * @return {?}
     */
    function (config) {
        if (config === void 0) { config = (/** @type {?} */ ({})); }
        return {
            ngModule: RouterMetaModule,
            providers: [
                config.configProvider || {
                    provide: ROUTE_META_CONFIG,
                    useValue: config.config,
                },
            ],
        };
    };
    RouterMetaModule.decorators = [
        { type: NgModule, args: [{},] }
    ];
    /** @nocollapse */
    RouterMetaModule.ctorParameters = function () { return [
        { type: RouterMetaModule, decorators: [{ type: Optional }, { type: SkipSelf }] },
        { type: RouterMetaService }
    ]; };
    return RouterMetaModule;
}());

export { ROUTE_META_CONFIG, RouterMetaContextService, RouterMetaModule, RouterMetaService, isDataWithMeta, unfoldContext };
//# sourceMappingURL=ngx-router-meta.js.map
